function T = adaptthresh(I,varargin)
 
     %{ 
 -- Function File: T = adaptthresh(I)
 -- Function File: T = adaptthresh(I,t)
 -- Function File: T = adaptthresh(I,t,options)
 -- Function File: T = adaptthresh(I,t,options)
    
    Given a gray image I and background and foreground pixels are
    segmented using the integral image of the input.
    
    The input t is the comparison (%), its default value is 15
    
    There is one option "neigh" to define the window size of the neighborhood pixels
    Its default value is 2*floor(size(I,1)/16)+1
    
    %}
    pkg load image
    t=15;
   
    if (nargin == 0 || nargin>4)
       error (usage());
   endif
   
     if (!isgray(I))
      error("multithresh: The input must be a gray image\n");
    endif
     NWZ= 2*floor(size(I,1)/16+1);  % neighborhood window size 
      % validate the comparison value
       loca=0; %is used to define the location of 'neigh' option
      if (nargin==2 || nargin==4)
        t=varargin{1};
         if (!isnumeric(t))
       error("adaptthresh: Comparison t % must be between 0 and 100\n");
       %elseif t<0 || t>100
       %error("adaptthresh: Comparison t % must be between 0 and 100\n");
       end
      end
      % neighborhood window size 
      if (nargin>2)
        if varargin{1+loca}!='neigh'
           error("adaptthresh:  'neigh' is used to  define the neighborhood size\n");
       end
       NWZ=varargin{nargin-1};
    
     if (!isnumeric (NWZ))
       error("multithresh: neighborhood window size must be integral, single, postive and odd\n");
       elseif size(NWZ)>1 || NWZ-floor(NWZ)!=0 || NWZ<0 || mod(NWZ,2)!=0
       error("multithresh: neighborhood window size must be integral, single, postive and odd\n");
       elseif NWZ>size(I,1) ||  NWZ>size(I,2)
       error("multithresh: neighborhood window size must be be smaller than\n the height and the width of the image\n");
     end
   end
   % convert image to double
   I=im2double(I);
   s=size(I);
   % extend the image to treat the boundaries of the image 
   I=padarray(I,[floor(NWZ/2),floor(NWZ/2)],'symmetric','both');
   % calculate the the integral image
   intImg=cumsum(cumsum(I),2);
   % create filter 
   Scalar=(100-t)/100;
   F=zeros(NWZ,NWZ);
   F(1,1)=1/NWZ^2*Scalar;
   F(end,end)=1/NWZ^2*Scalar;
   F(end,1)=-1/NWZ^2*Scalar;
   F(1,end)=-1/NWZ^2*Scalar;
   % Calculate threshold 
    T=conv2(intImg,F,'same');
    T=T(floor(NWZ/2)+1:end-floor(NWZ/2),floor(NWZ/2)+1:end-floor(NWZ/2));
endfunction

function msg=usage ()
  line1="adaptthresh: Invalid call to adaptthresh.  Correct usage is\n";
  line2="  -- Function File: T = adaptthresh(I)\n ";
  line3=" -- Function File: T = adaptthresh(I,t)\n ";
  line4=" -- Function File: T = adaptthresh(I,t,options)\n ";
  line5=" -- Function File: T = adaptthresh(I,options)\n\n ";
  line6="   I is a gray image\n";
  line7="    t must be between 0 and 100\n";
  line8="    options 'neigh' is used to  define the neighborhood size \n";
  msg=[line1,line2,line3,line4,line5,line6,line7,line8,"\n"];
endfunction
%------------------------------------------------------------------------