## -*- texinfo -*-
## @deftypefn  {Function File} {} colorcloud ( @var{rgb})
## @deftypefnx {Function File} {} colorcloud (@var{rgb}, @var{colorspace}) 
## @deftypefnx {Function File} {} colorcloud (@dots{}, @var{name},@var{value})
## @end deftypefn

function colorcloud(rgb,colorspace='rgb',varargin)
  if (nargin < 1 || nargin > 4)
    print_usage ();
   elseif (nargin>2)
    error("colorcloud(___,Name,Value) is not implemented yet \n") 
   elseif (!isrgb (rgb))
    error("The image must be an RGB image \n");
   else
    colorspaceValidation (colorspace);
  endif
   
  rgb=im2double(rgb);
  R=rgb(:,:,1)(:);
  G=rgb(:,:,2)(:);
  B=rgb(:,:,3)(:);
  % reduce number of points
  A=[[R*0.8+0.1]+[G*0.8+0.1]*100+[B*0.8+0.1]*1000];
  [~, index,~] = unique (A);
  R=R(index);  G=G(index);  B=B(index);
  hFigure=figure('color', [0.9, 0.9, 0.9]);
  ax1= axes('Position', [0, 0.1, 0.7,0.7]);
   
  rgbDrawPoints( R,G,B)
  set(hFigure, 'ToolBar', 'none');
   
  rotate3d (hFigure, "on")
  ax2 = axes('Position',[0.75 0.1 .2 .2])
  plotAxis()
  axis off

  addlistener (ax2, "view", {@my_listener,[ax2,ax1]})
  addlistener (ax1, "view", {@my_listener,[ax1,ax2]})
  set(gca, 'CameraViewAngleMode', 'manual');
endfunction

function check = isrgb (I)
  
  %Types={'single ',' double ',' int8 ',' int16 ',' int32 ',' uint8 ',' uint16 ',' uint32 '};
 
  check  = ((isnumeric (I) || isinteger (I) || isfloat(I)) && ! issparse (I) && ! isempty (I) && isreal (I) && ndims(I)==3);
endfunction

function check =colorspaceValidation (colorspace)
  
  switch colorspace
    case "rgb"
      colorspace = "rgb";
    case  {'hsv' , 'ycbcr' , 'lab'}
      error ([colorspace," colorspace is not implemented yet\n"])
      
    otherwise
     error ("colorspace must be  'rgb' | 'hsv' | 'ycbcr' | 'lab' \n")
  endswitch
  check = true;
 end
 
 function rgbDrawPoints(R,G,B)
   scatter3 (R, G, B, 5, [R G B],'.');
   %DrawBox
   B=[0 0 1 1 0;0 1 1 0 0;0 0 0 0 0];
   hold on
   plot3(B(1,:),B(2,:),B(3,:),'k')
   plot3(B(1,:),B(2,:),B(3,:)+1,'k')
   plot3(B(1,:),B(3,:),B(2,:),'k')
   plot3(B(1,:),B(3,:)+1,B(2,:),'k')
   %plotAxis()
   hold off
   axis off
   axis equal
   view(15,15)
 endfunction
 
 function plotAxis()
    plot3([0 1],[0 0],[0 0],'k','linewidth',1)
    text(1.2,-0.1,0,"R",'color','r')
    hold on
    plot3([0 0],[0 1],[0 0],'k')
    text(-0.1,1.2,0,"G",'color',[0.1 1 0.1])
    plot3([0 0],[0 0],[0 1],'k')
    text(-0.1,0,1.2,"B",'color','B')
    hold off
    view(15,15)
 endfunction

 function my_listener(h,dummy,F)
   ax2=F(1);
   ax1=F(2);
   View=get(ax2,'view');
   set(ax1,'view',View);
 endfunction