function output=demosaic(I,sensorAlignment)
 %{
  -- Function File: RGB = demosaic(I)
  -- Function File: RGB = demosaic(I,sensorAlignment) 
 
   demosaic is a function used to reconstruct a full color
   image from the incomplete color image. There are two inputs:
   I is the input image
   sensorAlignment is the function options which can be:
   'gbrg' or 'grbg' or 'bggr' or 'rggb' 
   the default value of  sensorAlignment is 'bggr'
   %}
  if nargin<1 ||  nargin>2 
   error (usage());
   end
  if ndims(I)~=2
  error('demosaic: the image must have only 2 dimensions')

  end
    
  if nargin==1
    sensorAlignment='bggr';
  end
    
    % Make the size of I even
    s=size(I);
    md=mod(s,2);
    I(s(1)+md(1),s(2)+md(2))=0;
    switch sensorAlignment
   case 'bggr'
     % Create CFA filter,  BGGR format
      cfaR = repmat([0 0;0 1],size(I)/2);
      cfaG = repmat([0 1;1 0],size(I)/2);
      cfaB = repmat([1 0;0 0],size(I)/2);
   case 'rggb'
       % Create CFA filter, RGGB format
      cfaR = repmat([1 0;0 0],size(I)/2);
      cfaG = repmat([0 1;1 0],size(I)/2);
      cfaB = repmat([0 0;0 1],size(I)/2);
    case 'grbg'
       % Create CFA filter, in GRBG format
      cfaR = repmat([0 1;0 0],size(I)/2);
      cfaG = repmat([1 0;0 1],size(I)/2);
      cfaB = repmat([0 0;1 0],size(I)/2);
      
     case 'gbrg'
       % Create CFA filter, BRG format
      cfaR = repmat([0 0;1 0],size(I)/2);
      cfaG = repmat([1 0;0 1],size(I)/2);
      cfaB = repmat([0 1;0 0],size(I)/2);
    ...
   otherwise
      error(["demosaic: the interpolation options must be:\n",...
      "'gbrg' or 'grbg' or 'bggr' or 'rggb' \n"])
end
             
% initial the three color channels
Rh = I.*cfaR;
Gh = I.*cfaG;
Bh = I.*cfaB;

% Creating 5x5 filters
% G at R and B locations
F_g_at_r_b=[0 0 -1 0 0,
            0 0 +2 0 0,
            -1 2 4 2 -1,
             0 0 +2 0 0,
             0 0 -1 0 0]/8;
% R at green in R row, B column. 
% The same filter is applied for B at green in B row, R column 
F_r_at_g_rb=[0 0 0.5 0 0,
            0 -1 0 -1 0,
            -1 4 5 4 -1,
             0 -1 0 -1 0,
             0 0 0.5 0 0]/8;
             
% R at green in B row, R column. 
% The same filter is applied for B at green in R row, B column 
F_r_at_g_br=F_r_at_g_rb';

% R at blue in B row, B column
% The same filter is applied for B at red in R row, R column
F_r_at_g_bb=[0 0 -1.5 0 0,
            0 2 0 2 0,
            -1.5 0 6 0 -1.5,
             0 2 0 2 0,
             0 0 -1.5 0 0]/8;
             
% applying the filter for  G at R and B locations            
IF_1_2=  conv2(I,F_g_at_r_b,'same') ;
 % applying the filter for  R at green in R row, B column   
IF_3_6=conv2(I,F_r_at_g_rb,'same') ;
% applying the filter for  R at green in B row, R column 
IF_4_7=conv2(I,F_r_at_g_br,'same');
% applying the filter for  R at green in B row, B column 
IF_5_8=conv2(I,F_r_at_g_bb,'same');

% find the locations of the calculated colors;
G=Gh;
R=Rh;
B=Bh;
% find locations of G at R and B locations 
K_1_2=find(Gh==0);
G(K_1_2)=IF_1_2(K_1_2);
% find locations of  R at green in R row, B column
M=size(cfaR);
cfaRL=[zeros(M(1),1),cfaR(:,1:end-1)];
cfaRR=[cfaR(:,2:end),zeros(M(1),1)];
K_3_7=find(cfaG==1 & cfaRL==1 & cfaRR==1);
% Fill the empty locations
R(K_3_7)=IF_3_6(K_3_7);
B(K_3_7)=IF_4_7(K_3_7);
% find locations of  R at green in B row, R column
cfaRT=[zeros(1,M(2));cfaR(1:end-1,:)];
cfaRB=[cfaR(2:end,:);zeros(1,M(2))];
K_4_6=find(cfaG==1 & cfaRT==1 & cfaRB==1);
% Fill the empty locations
R(K_4_6)=IF_4_7(K_4_6);
B(K_4_6)=IF_3_6(K_4_6);

%% locations of R at B
K_5=find(cfaB==1);
% Fill the empty locations
R(K_5)=IF_5_8(K_5);
%% locations of B at R
K_8=find(cfaR==1);
% Fill the empty locations
B(K_8)=IF_5_8(K_8);
%% produce the Orginal size
R=R(1:s(1),1:s(2));
B=B(1:s(1),1:s(2));
G=G(1:s(1),1:s(2));
output(:,:,1)=R; output(:,:,2)=G; output(:,:,3)=B;
end
%------------------------------------------------------------------------
function msg=usage ()
  line1="demosaic: Invalid call to demosaic.  Correct usage is:\n";
  line2=" -- Function File: RGB = demosaic(I)\n ";
  line3=" -- Function File: RGB = demosaic(I,sensorAlignment)\n ";
  msg=[line1,line2,line3,"\n"];
endfunction
%------------------------------------------------------------------------