## -*- texinfo -*-
## @deftypefn  {Function File} {} imresize3 (@var{V}, @var{scale})
## @deftypefnx {Function File} {} imresize3 (@var{V}, [@var{M} @var{N} @var{k}])
## @deftypefnx {Function File} {} imresize3 (@dots{}, @var{method})
## @end deftypefn
function B = imresize3(V, scale, method = "cubic")
pkg load image
  if (nargin < 2 || nargin > 3)
    print_usage ();
  elseif (! ImageValidation(V))
    error ("V must be 3D image and single | double | int8 | int16 | int32 | uint8 | uint16 | uint32 \n")
  elseif ((length(scale)!=1&& length(scale)!=3) ||! isnumeric (scale) || any (scale <= 0)|| any (isnan (scale)))
    error ("SCALE or [M N K] must be positive values\n")
  else
      method =  regexprep(tolower (method), '\s+', '');
      MethodValidation (method);
    %
  endif
  
 if length(scale)==1
   M=size(V,1)*scale;
   N=size(V,2)*scale;
   K=size(V,3)*scale;
 else
   M=scale(1);
   N=scale(2);
   K=scale(3);
   end
  
  
  B=imresize(V,[M N],'cubic');
  
  B=permute(B,[3 1 2]);

  B=imresize(B,[K M],'cubic');
  
  B=permute(B,[2 3 1]);
  
end

function check = ImageValidation (I)
  
  %Types={'single ',' double ',' int8 ',' int16 ',' int32 ',' uint8 ',' uint16 ',' uint32 '};
 
  check  = ((isnumeric (I) || isinteger (I) || isfloat(I)) && ! issparse (I) && ! isempty (I) && isreal (I) && ndims(I)==3);
endfunction

function check = MethodValidation (method)
  switch method
    case "cubic"
      method = "cubic";
    case  {"nearest" ,"linear", "box", "triangle", "lanczos2", "lanczos3"}
      error ([method," Method is not implemented yet\n"])
    otherwise
      error ("METHOD must be 'nearest' | 'linear' | 'cubic'| 'box' | 'triangle' | 'lanczos2' | 'lanczos3'\n")
  endswitch
endfunction
