## -*- texinfo -*-
## @deftypefn  {Function File} {} B=imrotate3 (@var{V}, @var{angle}, @var{W})
## @deftypefnx {Function File} {} B=imrotate3 (@var{V}, @var{angle}, @var{W}, @var{method}) 
## @deftypefnx {Function File} {} B=imrotate3 (@var{V}, @var{angle}, @var{W}, @var{method},@var{bbox}) 
## @deftypefnx {Function File} {} B=imrotate3 (@dots{}, 'FillValues',@var{fillValues})
## @end deftypefn

function B = imrotate3(V, angle, W, method = "linear", bbox = "loose", FillValuesName = 'FillValues', fillValues = 0)

  if (nargin < 3 || nargin > 7)
    print_usage ();
  end
  method =  regexprep(tolower (method), '\s+', '');
  if (! ImageValidation(V))
    error ("V must be 3D image and single | double | int8 | int16 | int32 | uint8 | uint16 | uint32 \n")
  elseif (!AngleValidation (angle))
    error("imrotate3: angle must be a scalar");
  elseif  (!VectorValidation (W))
    error ("imrotate3: W must be 3D vector single | double | int8 | int16 | int32 | uint8 | uint16 | uint32 \n")
  elseif (!MethodValidation (method))
    error ("METHOD must be 'nearest' | 'linear' | 'cubic' \n");
  elseif (! ischar (bbox) || ! any (strcmpi (bbox, {"loose", "crop"})))
    error("imrotate3: BBOX must be 'loose' or 'crop'\n");
  elseif  regexprep(tolower (FillValuesName), '\s+', '')!='fillvalues'
    error("imrotate3: FillValues property must be 'fillvalues'\n");
  elseif  (!fillValuesValidation (fillValues))
    error ("fillValues must be  single | double | int8 | int16 | int32 | uint8 | uint16 | uint32 \n")
  endif
  
  
  angle = mod( angle, 360);
  if angle == 0
    B=V; % no need for rotation
  endif
  % size of the original
  sizeV=size(V);
  % 3d coordinates  of the image V
  [XV,YV,ZV] = meshgrid(1:sizeV(2),1:sizeV(1),1:sizeV(3));
  % calculate the Rotation matrix using the Rodrigues rotation formula 
  R=rotationMatrix(W,angle);
  % corners of the matrix V
  [Xc,Yc,Zc] = meshgrid([0 sizeV(2)-1] ,[0 sizeV(1)-1],[0 sizeV(3)-1]);
  % Vc =([sizeV(2);sizeV(1);sizeV(3)]+1)/2;    
   % [Xc,Yc,Zc] = meshgrid([1 sizeV(2)] -Vc(1) ,[1 sizeV(1)]-Vc(2),[1 sizeV(3)]-Vc(3));
  cornersV=[Xc(:) Yc(:) Zc(:)];
  
  % find the size of the new Image B
  cornersB=R*cornersV';
   
  sB=ceil(max(cornersB')-min(cornersB'))+1;
  sizeB(2)=sB(1); 
  sizeB(1)=sB(2);
  sizeB(3)=sB(3);
  
  % find the transfer matrix 
  Vc =([sizeV(2);sizeV(1);sizeV(3)]+1)/2 ;         % centre of V 
  Bc =([sizeB(2);sizeB(1);sizeB(3)]+1)/2 ;         % centre of B
  d = Bc - R * Vc    ;       % tanslation between two center 
  T = [[R; 0 0 0] [d; 1]] ;  %transfer matrix 
  %3d coordinates  of the image B
  [X1,Y1,Z1] = meshgrid(1:sizeB(2),1:sizeB(1),1:sizeB(3));
  [sx,sy,sz] = size(X1);
  Z1=Z1(:);
  Bd = [X1(:), Y1(:), Z1, ones(length(Z1), 1)]';       %Vd is the 3d coordinates  of the image V
  % find the projection of the new image B on the original image V
  PT = inv(T)*Bd;
  XB = PT(1,:)./PT(4,:);
  YB = PT(2,:)./PT(4,:);
  ZB = PT(3,:)./PT(4,:);
    
  XB = reshape(XB, sx, sy,sz);
  YB = reshape(YB, sx, sy,sz);
  ZB = reshape(ZB, sx, sy,sz);
    
  B = interp3 (XV,YV,ZV,double(V),XB,YB,ZB);
  kk = find(isnan(B));
  B(kk) = fillValues;
  B = cast (B, class (V));
end

function check = ImageValidation (I)
  
  %Types={'single ',' double ',' int8 ',' int16 ',' int32 ',' uint8 ',' uint16 ',' uint32 '};
 
  check  = ((isnumeric (I) || isinteger (I) || isfloat(I)) && ! issparse (I) && ! isempty (I) && isreal (I) && ndims(I)==3);
endfunction

function check = AngleValidation (angle)
 
  %Types={'single ',' double ',' int8 ',' int16 ',' int32 ',' uint8 ',' uint16 ',' uint32 '};

  check  = ((isnumeric (angle) || isinteger (angle) || isfloat(angle)) && ! issparse (angle) && ! isempty (angle) && isreal (angle) && sum(size(angle))==2);
endfunction

function check = MethodValidation (method)
  
  switch method
    case "linear"
      method = "linear";
    case  {"nearest" ,"cubic"}
      error ([method," Method is not implemented yet\n"])
      
    otherwise
     error ("METHOD must be  'nearest' | 'linear' | 'cubic' \n")
  endswitch
  check= true;
end
 
function check = fillValuesValidation (fillValues)
  
  %Types={'single ',' double ',' int8 ',' int16 ',' int32 ',' uint8 ',' uint16 ',' uint32 '};
 
  check  = ((isnumeric (fillValues) || isinteger (fillValues) || isfloat(fillValues)) && ! issparse (fillValues) && ! isempty (fillValues) && isreal (fillValues) &&  sum(size(fillValues))==2);
endfunction

function check = VectorValidation (W)
 
  %Types={'single ',' double ',' int8 ',' int16 ',' int32 ',' uint8 ',' uint16 ',' uint32 '};
  if size(W,1)!=1 ||  size(W,2)!=3 || W*W'==0
      error("imrotate3:W must be 3D vector \n");
  endif

  check  = ((isnumeric (W) || isinteger (W) || isfloat(W)) && ! issparse (W) && ! isempty (W) && isreal (W) && !any(isnan(W)));
endfunction

function R=rotationMatrix(u,theta)
  
  theta=theta*pi/180;
  u=u/sqrt(u*u');

  W=[0 -u(3) u(2);u(3) 0 -u(1);-u(2) u(1) 0];

  %W=[0 -u(3) u(1);u(3) 0 -u(2);-u(1) u(2) 0];
   
  R=eye(3)+sin(theta)*W+(1-cos(theta))*W^2;
  
  R=R';
  
end
